{*******************************************************************************
   Unit
      TiledBmp.pas
   Description:
      TTiledBmp - the very stripped out revrite of TImage with ability to tile bitmap
   Versions:
   	'99
   Autor(s):
      Dimitry Statilko - support@sedittools.com**
   URL:
      http://www.sedittools.com
   Comments:
	History:
      '99    - July 1999
               .Fixed transparenct bug (Thanks to Thomas von Stetten)
      1.2('99 beta 1)-  March 1999
               .Transparency
      1.1   -  16/12/1998
               Canvas.Paint changed to BitBlt
      1.1	-	18/11/1998
               TsGlyphList support
      1.0 	- 	End of September 1998
      			Initial release
*******************************************************************************}
unit TiledBmp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  sGlyphsList, sConsts, sGraphics, stdUtils;

type
	TWallpaperMode = (wpTile, wpCenter, wpTopLeft);

  	TTiledBmp = class(TGraphicControl)
  	private
    	FImage: TBitmap;
    	FMode: TWallpaperMode;
      FGlyphList: TsGlyphList;
      FGlyphId: Integer;
      FTransparent: Boolean;
    	procedure PictureChanged(Sender: TObject);
      procedure SetGlyphList(Value: TsGlyphList);
      procedure SetGlyphId(Value: Integer);
      procedure SetImage(Value: TBitMap);
      function IsGlyphStored: Boolean;
      procedure SetMode(value: TWallpaperMode);
      procedure SetTransparent(Value: Boolean);
      procedure STMGlyphIdChanged(var Message: TMessage); message STM_GLYPHIDCHANGED;
	protected
      procedure RefreshTransparent; dynamic; // TVS99:
      procedure Resize; override; // TVS99:
    	procedure Paint; override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      constructor Create(AOwner: TComponent); override;
    	destructor Destroy; override;
   published
    	property Align default alClient;
      property GlyphList: TsGlyphList read FGlyphList write SetGlyphList;
      property GlyphListId: Integer read FGlyphId write SetGlyphId default -1;
      property Picture: TBitMap read FImage write SetImage stored IsGlyphStored;
    	property Mode: TWallpaperMode read FMode write SetMode;
      property Transparent: Boolean read FTransparent write SetTransparent default FALSE;
      property Visible;
  	end;

procedure Register;

implementation

{.$R *.dcr}

constructor TTiledBmp.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   ControlStyle := ControlStyle + [csReplicatable];
   FGlyphList := nil;
   FGlyphId := -1;
   FImage := TBitmap.Create;
   FImage.OnChange := PictureChanged;
   Align := alClient;
   FTransparent := False;
end;

destructor TTiledBmp.Destroy;
begin
  	FImage.Free;
  	inherited;
end;

procedure TileBitmap(Bitmap: TBitmap; Canvas: TCanvas; Width, Height: Integer; transparent: Boolean);
var
	ii, jj: Integer;
   w, h: Integer;
   cl: TColor;
begin
   w := Bitmap.Width;
   h := Bitmap.Height;
   cl := clNone; 
   if Transparent then
      cl := Bitmap.TransparentColor;
  	for ii := 0 to (Width-1) div w do
  		for jj := 0 to (Height-1) div h do
         if Transparent then
            DrawBitmapTransparent( Canvas, ii*w, jj*h, Bitmap, cl)
         else
            BitBlt( Canvas.handle, ii*w, jj*h, w, h, Bitmap.canvas.handle, 0, 0, srccopy);
end;

// TVS99: New Procedure for repainting the Parent-Control.
procedure TTiledBmp.RefreshTransparent;
var
   R: TRect;
begin
   if FTransparent then begin
      if (Parent <> nil) and (Parent.Handle <> 0) then begin
         R := ClientRect;
         InvalidateRect(Parent.Handle, @R, TRUE);
      end;
   end else
      Invalidate;
end;

// TVS99: Override default Procedure.
procedure TTiledBmp.Resize;
begin
   inherited Resize;
   RefreshTransparent; // TVS99: Refresh background if transparent mode
end;

procedure TTiledBmp.Paint;
var
	Dest: TRect;
begin
   if not FImage.Empty then begin
 	  if FMode = wpTile then
   		TileBitmap( FImage, inherited Canvas, Width, Height, FTransparent)
   	else begin
   		if FMode = wpCenter then
   			Dest := Bounds((Width - FImage.Width) div 2, (Height - FImage.Height) div 2,
      			FImage.Width, FImage.Height)
      	else
				Dest := Rect(0, 0, FImage.Width, FImage.Height);
         if FTransparent then
            StretchBitmapRectTransparent(inherited Canvas, Dest.Left, Dest.Top, WidthOf(Dest), HeightOf(Dest),
               Rect(0, 0, FImage.Width, FImage.Height), FImage, FImage.TransparentColor)
         else with inherited Canvas do
    			StretchDraw(Dest, FImage);
  	   end;
   end;
   if csDesigning in ComponentState then with inherited Canvas do begin
   	Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
   end;
end;

procedure TTiledBmp.Notification(AComponent: TComponent; Operation: TOperation);
begin
	inherited;
   if (AComponent = FGlyphList) and (Operation = opRemove) then begin
   	FGlyphList := nil;
      Invalidate;
   end;
end;

procedure TTiledBmp.STMGlyphIdChanged(var Message: TMessage);
begin
	if Message.WParam = FGlyphId then begin
      if Message.lParam = Message.wParam then
			PictureChanged(self)
      else
			GlyphListId := Message.LParam;
   end;
end;

procedure TTiledBmp.SetImage(Value: TBitmap);
begin
  	FImage.Assign(Value);
   FGlyphId := -1;
   RefreshTransparent; //TVS 99
end;

function TTiledBmp.IsGlyphStored: Boolean;
begin
   Result := FGlyphId = -1;
end;

procedure TTiledBmp.SetGlyphList(Value: TsGlyphList);
begin
	if FGlyphList <> Value then begin
      if FGlyphList <> nil then
			FGlyphList.ChangeNotification(self, FALSE);
   	FGlyphList := Value;
      if FGlyphList <> nil then begin
			FGlyphList.ChangeNotification(self, TRUE);
         if FGlyphId > -1 then
            FImage.Assign(FGlyphList[FGlyphId]);
      end else
         FImage.Assign(nil);
   end;
end;

procedure TTiledBmp.SetGlyphId(Value: Integer);
begin
	if FGlyphId <> Value then begin
		if csLoading in ComponentState then
   		FGlyphId := Value
		else begin
   		if FGlyphList = nil then
   			Exit;
      	if (Value > -1) then begin
            if FGlyphList[Value] = nil then
               Raise Exception.CreateFmt(SErrorGlyphNotFound, [Value]);
            if not FGlyphList[Value].InheritsFrom(TBitMap) then
      		   Raise Exception.Create(SErrorNeedsBmp);
         end;
   		FGlyphId := Value;
      	FImage.Assign(FGlyphList[FGlyphId]);
      end;
   end;
end;

procedure TTiledBmp.PictureChanged( Sender: TObject);
begin
   if ((FMode = wpTile) and not FImage.Empty)  or ((FImage.width >= Width) and (FImage.Height >= Height)) then
      ControlStyle := ControlStyle + [csOpaque]
   else
      ControlStyle := ControlStyle - [csOpaque];
  	Invalidate;
end;

procedure TTiledBmp.SetMode(value: TWallpaperMode);
begin
	if Value <> FMode then begin
   	FMode := Value;
      RefreshTransparent; //TVS 99
   end;
end;

procedure TTiledBmp.SetTransparent(Value: Boolean);
begin
	if Value <> FTransparent then begin
   	FTransparent := Value;
      RefreshTransparent; //TVS 99
   end;
end;

procedure Register;
begin
   RegisterComponents(SEditToolsPageCaption, [TTiledBmp]);
end;

end.
